@@ -4,9 +4,9 @@
 #' structures as [`dht`][mrds::dht]. Note that only geographical stratification
 #' as supported in `dht` is allowed.
 #'
-#' @param model a model fitted by [`ds`][ds] or a list of models
-#' @param flatfile Data provided in the flatfile format. See
-#' [`flatfile`][flatfile] for details.
+#' @param model a model fitted by [`ds`] or a list of models
+#' @param flatfile Data provided in the flatfile format. See [`flatfile`] for
+#' details.
 #' @param convert.units conversion between units for abundance estimation, see
 #' "Units", below. (Defaults to 1, implying all of the units are "correct"
 #' already.) This takes precedence over any unit conversion stored in `model`.
@@ -19,8 +19,7 @@
 #' @param nboot number of bootstrap replicates
 #' @param summary_fun function that is used to obtain summary statistics from
 #' the bootstrap, see Summary Functions below. By default
-#' [`bootdht_Nhat_summarize`][bootdht_Nhat_summarize] is used, which just
-#' extracts abundance estimates.
+#' [`bootdht_Nhat_summarize`] is used, which just extracts abundance estimates.
 #' @param select_adjustments select the number of adjustments in each
 #' bootstrap, when `FALSE` the exact detection function specified in `model` is
 #' fitted to each replicate. Setting this option to `TRUE` can significantly
@@ -28,6 +27,7 @@
 #' must have been fitted with `adjustment!=NULL`.
 #' @param sample_fraction what proportion of the transects was covered (e.g.,
 #' 0.5 for one-sided line transects).
+#' @param multipliers `list` of multipliers. See "Multipliers" below.
 #' @param progress_bar which progress bar should be used? Default "base" uses
 #' `txtProgressBar`, "none" suppresses output, "progress" uses the
 #' `progress` package, if installed.
@@ -44,6 +44,30 @@
 #' return any information within those objects, for example abundance or
 #' density estimates or the AIC for each model. See Examples below.
 #'
+#' @section Multipliers:
+#' It is often the case that we cannot measure distances to individuals or
+#' groups directly, but instead need to estimate distances to something they
+#' produce (e.g., for whales, their blows; for elephants their dung) -- this is
+#' referred to as indirect sampling. We may need to use estimates of production
+#' rate and decay rate for these estimates (in the case of dung or nests) or
+#' just production rates (in the case of songbird calls or whale blows). We
+#' refer to these conversions between "number of cues" and "number of animals"
+#' as "multipliers".
+#'
+#' The `multipliers` argument is a `list`, with 3 possible elements (`creation`
+#' and `decay`). Each element of which is either:
+#' * `data.frame` and must have at least a column named `rate`, which abundance
+#'    estimates will be divided by (the term "multiplier" is a misnomer, but
+#'    kept for compatibility with Distance for Windows). Additional columns can
+#'    be added to give the standard error and degrees of freedom for the rate
+#'    if known as `SE` and `df`, respectively. You can use a multirow
+#'    `data.frame` to have different rates for different geographical areas
+#'    (for example). In this case the rows need to have a column (or columns)
+#'    to `merge` with the data (for example `Region.Label`).
+#' * a `function` which will return a single estimate of the relevant
+#'   multiplier. See [`make_activity_fn`] for a helper function for use with the
+#'   `activity` package.
+#'
 #' @section Model selection:
 #' Model selection can be performed on a per-replicate basis within the
 #' bootstrap. This has three variations:
@@ -59,12 +83,13 @@
 #' This last option can be extremely time consuming.
 #'
 #' @section Parallelization:
-#' If `cores`>1 then the `parallel`/`doParallel`/`foreach` packages will be
-#' used to run the computation over multiple cores of the computer. To use this
-#' component you need to install those packages using:
-#' `install.packages(c("foreach", "doParallel"))` It is advised that you do not
-#' set `cores` to be greater than one less than the number of cores on your
-#' machine.
+#' If `cores`>1 then the `parallel`/`doParallel`/`foreach`/`doRNG` packages
+#' will be used to run the computation over multiple cores of the computer. To
+#' use this component you need to install those packages using:
+#' `install.packages(c("foreach", "doParallel", "doRNG"))` It is advised that
+#' you do not set `cores` to be greater than one less than the number of cores
+#' on your machine. The `doRNG` package is required to make analyses
+#' reproducible ([`set.seed`] can be used to ensure the same answers).
 #'
 #' It is also hard to debug any issues in `summary_fun` so it is best to run a
 #' small number of bootstraps first in parallel to check that things work. On
@@ -76,9 +101,8 @@
 #' @importFrom utils txtProgressBar setTxtProgressBar getTxtProgressBar
 #' @importFrom stats as.formula AIC
 #' @importFrom mrds ddf dht
-#' @seealso [`summary.dht_bootstrap`][summary.dht_bootstrap] for how to
-#' summarize the results, [`bootdht_Nhat_summarize`][bootdht_Nhat_summarize]
-#' for an example summary function.
+#' @seealso [`summary.dht_bootstrap`] for how to summarize the results,
+#' [`bootdht_Nhat_summarize`] for an example summary function.
 #' @export
 #' @examples
 #' \dontrun{
@@ -107,6 +131,7 @@ bootdht <- function(model,
                     convert.units=1,
                     select_adjustments=FALSE,
                     sample_fraction=1,
+                    multipliers=NULL,
                     progress_bar="base",
                     cores=1){
 
@@ -114,7 +139,7 @@ bootdht <- function(model,
     stop("At least one of resample_strata, resample_obs, resample_transects must be TRUE")
   }
 
-  # if we have a list...
+  # make everything a list...
   if(!all(class(model)=="list")){
     models <- list(model)
     # yes, I am a monster
@@ -126,13 +151,16 @@ bootdht <- function(model,
     convert.units <-  NULL
   }
 
+  # only use valid ds models
   for(i in seq_along(models)){
-    # only use valid ds models
     if(!all(class(models[[i]])=="dsmodel")){
       stop("Only models fitted by Distance::ds() may be used")
     }
   }
   dat <- flatfile
+  if(!("object" %in% names(dat))){
+    dat$object <- 1:nrow(dat)
+  }
 
   # if we're using the default summary function and have Area 0 then
   # we're not going to have a good time
@@ -143,6 +171,19 @@ bootdht <- function(model,
 
   # apply the sample fraction
   dat <- dht2_sample_fraction(sample_fraction, dat)
+  dat$Effort <- dat$Effort * dat$sample_fraction
+  dat$sample_fraction <- NULL
+
+  # process non-function multipliers
+  multipliers_nofun <- Filter(Negate(is.function), multipliers)
+  if(length(multipliers_nofun) > 0){
+    dat <- dht2_multipliers(multipliers_nofun, dat)
+  }else{
+    dat$rate <- 1
+  }
+
+  # get any multiplier functions
+  multipliers_fun <- Filter(is.function, multipliers)
 
   # this can be generalized later on
   stratum_label <- "Region.Label"
@@ -161,93 +202,13 @@ bootdht <- function(model,
     lpars <- as.list(model$call)
     for(i in seq(2, length(lpars))){
       if(is.symbol(model$call[[names(lpars)[i]]])){
-        model$call[[names(lpars)[i]]] <- eval(lpars[[i]])
+        model$call[[names(lpars)[i]]] <- eval(lpars[[i]],
+                                              envir=parent.frame(n=3))
       }
     }
     model
   })
 
-  # count failures
-  nbootfail <- 0
-  # function to do a single bootstrap iteration
-  bootit <- function(bootdat, models, our_resamples, summary_fun,
-                     convert.units, pb, ...){
-    # sample at the right levels
-    for(sample_thingo in our_resamples){
-      # what are the possible samples at this level
-      levs <- unique(dat[[sample_thingo]])
-      nlevs <- length(levs)
-      levs <- sample(levs, nlevs, replace=TRUE)
-
-      # make a new data frame with the correct number of replicates of the
-      # per-stratum data in it
-      bootdat <- lapply(levs, function(x){
-        bootdat[bootdat[[sample_thingo]] == x, ]
-      })
-      # make a special index to make unique IDs later
-      iind <- rep(seq_len(length(bootdat)), lapply(bootdat, nrow))
-      # make list of data.frames into one frame
-      bootdat <- do.call("rbind", bootdat)
-      # put that ID in there
-      bootdat[[paste0(sample_thingo, "_ID")]] <- iind
-    }
-
-    # need unique object IDs
-    bootdat$object <- seq_len(nrow(bootdat))
-    # get the sample labels right
-    bootdat$Sample.Label <- paste0(bootdat[[sample_label]], "-",
-                                   bootdat[[paste0(sample_thingo, "_ID")]])
-
-    aics <- rep(NA, length(models))
-    for(i in seq_along(models)){
-      model <- models[[i]]
-
-      # setup the call to ds
-      df_call <- model$call
-
-      # if we want the number of adjustments to be selected each iteration...
-      if(select_adjustments){
-        df_call$order <- NULL
-      }
-      # insert the new data into the model
-      df_call$data <- bootdat
-      if(!is.null(convert.units)){
-        df_call$convert.units <- convert.units
-      }
-
-      # fit that and update what's in models
-      models[[i]] <- try(suppressMessages(eval(df_call)),
-                         silent=TRUE)
-
-      if(any(class(models[[i]]) == "try-error")){
-        # if the model failed, return NA
-        aics[i] <- NA
-      }else{
-        # if that wasn't bad, grab the AIC
-        aics[i] <- AIC(models[[i]])$AIC
-      }
-    }
-
-    # update progress bar
-    pb$increment(pb$pb)
-
-    if(all(is.na(aics))){
-      # if no models fitted, return NA
-      nbootfail <<- nbootfail + 1
-      return(NA)
-    }else{
-      fit <- models[[which.min(aics)]]
-      # handle errors
-      if(any(class(fit) == "try-error") ||
-         any(is.na(fit$ddf$hessian))){
-        nbootfail <<- nbootfail + 1
-        return(NA)
-      }else{
-        return(summary_fun(fit$dht, fit$ddf))
-      }
-    }
-  }
-
   cat(paste0("Performing ", nboot, " bootstraps\n"))
 
   if(cores > 1 & progress_bar != "none"){
@@ -286,41 +247,68 @@ bootdht <- function(model,
     stop("progress_bar must be one of \"none\", \"base\" or \"progress\"")
   }
 
-  # run the code
+  # run the bootstrap
   if(cores > 1){
     if (!requireNamespace("foreach", quietly = TRUE) &
         !requireNamespace("doParallel", quietly = TRUE) &
+        !requireNamespace("doRNG", quietly = TRUE) &
         !requireNamespace("parallel", quietly = TRUE)){
       stop("Packages 'parallel', 'foreach' and 'doParallel' need to be installed to use multiple cores.")
     }
 
     # build the cluster
-    cl <- parallel::makeCluster(cores, outfile="")
+    cl <- parallel::makeCluster(cores)
     doParallel::registerDoParallel(cl)
+    # shutdown cluster when the function exits
+    # (this works even if the function crashes)
+    on.exit(parallel::stopCluster(cl))
+
+    # load the activity package in the other sessions
+    if(length(multipliers_fun) > 0){
+      packages <- c("activity")
+    }else{
+      packages <- NULL
+    }
 
     # needed to avoid a syntax error/check fail
-    `%dopar2%` <- foreach::`%dopar%`
+    `%dopar%` <- foreach::`%dopar%`
+    `%dorng2%` <- doRNG::`%dorng%`
     # fit the model nboot times over cores cores
     # note there is a bit of fiddling here with the progress bar to get it to
     # work (updates happen in this loop rather than in bootit)
-    boot_ests <- foreach::foreach(i=1:nboot) %dopar2% {
+    boot_ests <- foreach::foreach(i=1:nboot, .packages=packages) %dorng2% {
       bootit(dat, models=models, our_resamples=our_resamples,
              summary_fun=summary_fun, convert.units=convert.units,
-             pb=list(increment=function(pb){invisible()}))
+             pb=list(increment=function(pb){invisible()}),
+             multipliers_fun=multipliers_fun, sample_label=sample_label,
+             select_adjustments=select_adjustments)
     }
-    # shutdown cluster
-    parallel::stopCluster(cl)
-
-    # post-process
-    nbootfail <- sum(unlist(lapply(boot_ests, is.na)))
-    boot_ests <- Filter(function(x) !is.na(x), boot_ests)
 
   }else{
     boot_ests <- replicate(nboot,
                            bootit(dat, models=models, our_resamples,
                                   summary_fun, convert.units=convert.units,
-                                  pb=pb), simplify=FALSE)
+                                  pb=pb, multipliers_fun=multipliers_fun,
+                                  sample_label=sample_label,
+                                  select_adjustments=select_adjustments),
+                           simplify=FALSE)
   }
+
+  # do some post-processing
+  fail_fun <- function(x) class(x)=="bootstrap_failure"
+  # add replicate IDs
+  bootids <- seq_len(length(boot_ests))
+  # how many failures
+  failures <- length(Filter(fail_fun, boot_ests))
+  # remove failures from IDs
+  bootids <- as.list(bootids[unlist(lapply(boot_ests, Negate(fail_fun)))])
+  # get just the "good" results
+  boot_ests <- Filter(Negate(fail_fun), boot_ests)
+  # add IDs to estimates list
+  boot_ests <- mapply(cbind.data.frame,
+                      boot_ests, bootstrap_ID=bootids,
+                      SIMPLIFY=FALSE)
+
   # the above is then a list of thingos, do the "right" thing and assume
   # they are data.frames and then rbind them all together
   boot_ests <- do.call(rbind.data.frame, boot_ests)
@@ -328,7 +316,7 @@ bootdht <- function(model,
   cat("\n")
 
   attr(boot_ests, "nboot") <- nboot
-  attr(boot_ests, "nbootfail") <- nbootfail
-  class(boot_ests) <- "dht_bootstrap"
+  attr(boot_ests, "failures") <- failures
+  class(boot_ests) <- c("dht_bootstrap", "data.frame")
   return(boot_ests)
 }
