@@ -17,8 +17,8 @@ test_that("Input errors are thrown correctly",{
   expect_error(ds(),
                "argument \"data\" is missing, with no default")
   expect_error(ds(list()),
-               "Your data must \\(at least\\) have a column called 'distance'!")
-  
+               "Your data must \\(at least\\) have a column called 'distance' or 'distbegin' and 'distend'!")
+
   # incorrect data types
   expect_error(ds("distance"),
                "data is not a data.frame nor are the values numeric")
@@ -37,6 +37,10 @@ test_that("Input errors are thrown correctly",{
   expect_error(ds(egdata,4,cutpoints=c(2,3,4)),
                "The first cutpoint must be 0 or the left truncation distance!")
 
+  # uniform without adjustments
+  expect_error(ds(egdata,4,key="unif", adjustment=NULL),
+               "Can't use uniform key with no adjustments.")
+
   # uniform with covariates?
   expect_error(ds(egdata,4,key="unif",formula=~size),
                "Can't use uniform key with covariates.")
@@ -45,30 +49,40 @@ test_that("Input errors are thrown correctly",{
   expect_error(ds(egdata,4,key="hn",formula=~banana),
                "Variable\\(s\\): banana are in the model formula but not in the data.")
 
+  # observer is a reserved column
+  egdata$observer <- c("Fred", "Larry")
+  expect_error(ds(egdata,4,key="hn"),
+               "'observer' is a reserved column name, please rename this column.")
+  egdata$observer <- 1:3
+  expect_error(ds(egdata,4,key="hn"),
+               "'observer' is a reserved column name, please rename this column.")
 
 })
 
 
 test_that("Simple models work",{
-  
+  skip_on_cran()
+
   #Test that ds can deal with a numeric vector as input
-  distances <- c(1.02,0.89,0.21,1.83,0.09,1.34,2.1,0.98,1.8,0.32,0.83,1.6,0.92,0.66,0.31,0.55,1.13,0.5,0.46,1)
+  distances <- c(1.02, 0.89, 0.21, 1.83, 0.09, 1.34, 2.1, 0.98, 1.8, 0.32,
+                 0.83, 1.6, 0.92, 0.66, 0.31, 0.55, 1.13, 0.5, 0.46, 1)
   numeric.test <- suppressMessages(ds(distances))
   data.frame.test <- suppressMessages(ds(data.frame(distance = distances)))
-  #Passing in distances as a vector should be identical to passing them in a data.frame
+  # Passing in distances as a vector should be identical to passing them in a
+  # data.frame
   expect_equal(numeric.test$ddf$par, data.frame.test$ddf$par)
 
   # same model, but calculating abundance
   # need to supply the region, sample and observation tables
-  region<-book.tee.data$book.tee.region
-  samples<-book.tee.data$book.tee.samples
-  obs<-book.tee.data$book.tee.obs
+  region <- book.tee.data$book.tee.region
+  samples <- book.tee.data$book.tee.samples
+  obs <- book.tee.data$book.tee.obs
 
   egdata <- egdata[egdata$observer==1,]
 
   # half-normal key should get selected
   ds.dht.model <- suppressMessages(ds(egdata,4,region.table=region,
-               sample.table=samples,obs.table=obs))
+                                      sample.table=samples,obs.table=obs))
   # pars and lnl
   expect_equal(ds.dht.model$ddf$par, 0.6632435,tol=par.tol)
   expect_equal(ds.dht.model$ddf$lnl, -154.5692, tol=lnl.tol)
@@ -83,18 +97,17 @@ test_that("Simple models work",{
   #                            adj.order=2), data=egdata, method="ds",
   #              meta.data=list(width=4))
   tp <- c(0.66068510, -0.01592872)
-  names(tp) <- c("X.Intercept.","V2")
-  expect_equal(ds.model.cos2$ddf$par, tp, tol=par.tol)
+  expect_equal(unname(ds.model.cos2$ddf$par), tp, tol=par.tol)
   expect_equal(ds.model.cos2$ddf$lnl, -154.5619, tol=1e-6)
   expect_equal(ds.model.cos2$dht$individuals$N$Estimate[3], 642.9442, tol=N.tol)
 
   # specify order 2 and 4 cosine adjustments
-  ds.model.cos24<-suppressMessages(ds(egdata,4,adjustment="cos",order=c(2,4),
+  ds.model.cos24<-suppressMessages(suppressWarnings(ds(egdata,4,
+                     adjustment="cos",order=c(2,4),
                      region.table=region, sample.table=samples, obs.table=obs,
-                     monotonicity=FALSE))
+                     monotonicity=FALSE)))
   tp <- c(0.661391356, -0.008769883, -0.041465153)
-  names(tp) <- c("X.Intercept.","V2","V3")
-  expect_equal(ds.model.cos24$ddf$par, tp, tol=par.tol)
+  expect_equal(unname(ds.model.cos24$ddf$par), tp, tol=par.tol)
   expect_equal(ds.model.cos24$ddf$lnl, -154.5084, tol=lnl.tol)
   expect_equal(ds.model.cos24$dht$individuals$N$Estimate[3],620.0747,tol=N.tol)
 
@@ -114,6 +127,7 @@ test_that("Simple models work",{
 
 
 test_that("Uniform does work after all",{
+  skip_on_cran()
 
   egdata <- egdata[egdata$observer==1,]
 
@@ -126,62 +140,136 @@ test_that("Uniform does work after all",{
   dd <- suppressMessages(ds(egdata,4,key="unif",order=c(1,2)))
   expect_equal(dd$ddf$par, c(0.7050144, -0.1056291), tol=par.tol)
 
-
 })
 
 
-
 test_that("Truncation is handled",{
+  skip_on_cran()
 
   egdata <- egdata[egdata$observer==1,]
 
   # setting the truncation is correct
-  expect_equal(suppressMessages(ds(egdata,4,key="hn",order=0))$ddf$meta.data$width,4)
+  expect_equal(suppressMessages(ds(egdata, 4, key="hn",
+                                   order=0))$ddf$meta.data$width, 4)
 
   # largest observed distance
-  expect_equal(suppressMessages(ds(egdata,key="hn",order=0))$ddf$meta.data$width,3.84)
-
+  expect_equal(suppressMessages(ds(egdata, key="hn",
+                                   order=0))$ddf$meta.data$width, 3.84)
 
   # remove observations after 3.8
-  egdata <- egdata[egdata$distance <= 3.8,]
+  egdata <- egdata[egdata$distance <= 3.8, ]
 
   # largest cutpoint
   expect_equal(suppressMessages(ds(egdata,key="hn",order=0,
-                  cutpoints=c(0,1,2,3,3.8)))$ddf$meta.data$width,3.8)
+                             cutpoints=c(0,1,2,3,3.8)))$ddf$meta.data$width,3.8)
 
   # largest bin
-  bin.data <- Distance:::create.bins(egdata,c(0,1,2,3,3.8))
-  expect_equal(suppressMessages(ds(bin.data,key="hn",order=0))$ddf$meta.data$width,3.8)
+  bin.data <- create.bins(egdata,c(0,1,2,3,3.8))
+  expect_equal(suppressMessages(ds(bin.data, key="hn",
+                                   order=0))$ddf$meta.data$width, 3.8)
 
 })
 
 test_that("adjustments expand correctly",{
+  skip_on_cran()
 
   egdata <- egdata[egdata$observer==1,]
 
   # hn + cos(2)
-  expect_equal(summary(ds(egdata,4,key="hn",order=2))$ddf$name.message,
+  expect_equal(suppressWarnings(summary(ds(egdata, 4, key="hn",
+                                           order=2))$ddf$name.message),
                "half-normal key function with cosine(2) adjustments")
 
   # hn + cos(2,3)
-  expect_equal(summary(ds(egdata,4,key="hn",order=2:3))$ddf$name.message,
+  expect_equal(suppressWarnings(summary(ds(egdata, 4, key="hn",
+                                           order=2:3))$ddf$name.message),
                "half-normal key function with cosine(2,3) adjustments")
 
   # hn + cos(2,3,4,5)
-  expect_equal(summary(ds(egdata,4,key="hn",order=5))$ddf$name.message,
+  expect_equal(suppressWarnings(summary(ds(egdata, 4, key="hn",
+                                           order=5))$ddf$name.message),
                "half-normal key function with cosine(2,3,4,5) adjustments")
 
   #unif + cos(1,2)
-  expect_equal(summary(ds(egdata,4,key="unif",order=2))$ddf$name.message,
+  expect_equal(suppressWarnings(summary(ds(egdata, 4, key="unif",
+                                           order=2))$ddf$name.message),
                "uniform key function with cosine(1,2) adjustments")
 
 })
 
 # reported by Len Thomas 20 August
 test_that("Percentage truncation works when distances are missing",{
+  skip_on_cran()
 
   data(minke)
 
   expect_equal(ds(minke, truncation="15%", adjustment=NULL)$ddf$criterion,
                -8.1705496, tol=1e-5)
 })
+
+
+# just distend and distbegin can be supplied
+
+test_that("just distend and distbegin can be supplied", {
+  # make some data
+  bin.data <- create.bins(egdata, c(0, 1, 2, 3, 4))
+  bin.data$distance <- NULL
+  expect_message(ds.model <- ds(bin.data, 4, monotonicity=FALSE, key="hn",
+                                adjustment=NULL),
+                 "^Columns \"distbegin\" and \"distend\" in data: performing a binned analysis....*")
+})
+
+# max adjustments arg
+test_that("max.adjustments works",{
+  skip_on_cran()
+
+  egdata <- egdata[egdata$observer==1,]
+
+  # setting max.adjustments=0 gives no adjustments
+  expect_equal(summary(ds(egdata, 4, key="hn", max.adjustments=0,
+                          adjustment="cos"))$ddf$name.message,
+               "half-normal key function")
+
+  # some delicious stake
+  data(stake77)
+  dists <- stake77$PD[stake77$Obs2==1]
+  dists <- c(dists, dists[dists>10])
+  dists <- c(dists, dists[dists<5])
+  dists <- c(dists, dists[dists<5])
+
+  # ignore warnings below from monotonicity checks, don't care about that here
+  expect_equal(summary(suppressWarnings(
+                ds(dists, 20, key="hn", max.adjustments=3,
+                   adjustment="cos")))$ddf$name.message,
+               "half-normal key function with cosine(2,3,4) adjustments")
+
+  expect_equal(summary(suppressWarnings(
+                ds(dists, 20, key="hn", max.adjustments=2,
+                   adjustment="cos")))$ddf$name.message,
+               "half-normal key function with cosine(2,3) adjustments")
+
+  expect_equal(summary(suppressWarnings(
+                ds(dists, 20, key="hn", max.adjustments=1,
+                   adjustment="cos")))$ddf$name.message,
+               "half-normal key function with cosine(2) adjustments")
+
+  expect_equal(summary(suppressWarnings(
+                ds(dists, 20, key="hn", max.adjustments=6,
+                   adjustment="cos")))$ddf$name.message,
+               "half-normal key function with cosine(2,3,4) adjustments")
+})
+
+# warnings when bad models get fitted
+
+test_that("warnings of bad models get thrown",{
+  skip_on_cran()
+
+  # data with spike for which hazard-rate estimates
+  # 0 shape par
+  # based on data posted on distance list but rescaled and edited for anonymity
+  dat <- c(0, 0, 0, 0, 0.008, 0.008, 0.013, 0.022, 0.027, 0.05, 0.058, 0.137,
+           0.172, 0.2, 0.2, 0.2, 0.2, 0.25, 0.26, 0.29, 0.34, 0.36)
+  expect_warning(ds(dat, key="hr", adjustment=NULL),
+               "Estimated hazard-rate scale parameter close to 0 \\(on log scale\\). Possible problem in data \\(e.g., spike near zero distance\\).")
+
+})
